@@ -13,7 +13,7 @@
 
 namespace NMix {
 
-const int _MCMC_max_dim = 3;                            /** maximal dimension of the response vector **/
+const int _MCMC_max_dim = 3;                            /** maximal dimension of the response vector when RJ-MCMC is to be used **/
 
 enum _NMix_type_priorK {K_FIXED, K_UNIF, K_TPOISS};     /** possible prior distributions for K                         **/   
 enum _NMix_type_priormuQ {MUQ_NC, MUQ_IC};              /** possible prior distributions for mean and inverse variance **/
